#include<reg51.h>
unsigned char name;			//Variable to copy from SBUF
unsigned int i=0, j=0;		//Variables for software interrupt
sfr dat=0xA0;				//Defining dat as Port 2. Here we are giving a different name to Port 2 as dat. 
							// Such definitions are done in the header file reg51.h; there is a definition for port 2 in that 							//file as sfr P2=0xA0.
sbit rs=P1^0;				//Defining rs as P1.0 pin. Pin on LCD to indicate instruction or data.
sbit rw=P1^1;				//Defining rw as P1.1 pin. Pin on LCD to indicate read or write.
sbit en=P1^2;				//Defining en as P1.2 pin. Pin on LCD to latch the data on the data pins.
sbit busy=P2^7;			//Defining busy as P2.7 pin. Pin on LCD to indicate LCD is ready or busy.
void ready()
void ready()				//Function to check if LCD is busy or ready.
{
	busy=1;				//Making the P2.7 pin as input pin by writing a '1' on it.
	rs=0;					//Indicates instruction and not data
	rw=1;					//Indicates read and not write
	en=0;					//A low to high going pulse on en pin.
	en=1;
	while(busy==1)			//Wait till the LCD is busy.
	{
		en=0;
		for(i=0;i<25;i++);
		en=1;
	}
}
void lcdcommand(char cmd)		//Function to write the command to LCD
{
	
	dat=cmd;					//Write the command on the Port 2 so as to issue it to LCD.
	rs=0;						//Indicates instruction.
	rw=0;						//Indicates Write.
	en=1;						//A high to low pulse on en pin to latch the command
	for(i=0;i<125;i++);
	en=0;
}
void lcddata(char d)				//Function to write data to LCD
{
	ready();					//Check if the LCD is ready by calling the ready function.
	dat=d;						//Write the data on the Port 2 so that it is given to the LCD.
	rs=1;						//Indicates data.
	rw=0;						//Indicates write.
	en=1;						//A high to low pulse on en pin to latch the command
	for(i=0;i<25;i++);
	en=0;
}
void serialISR(void) interrupt 4
{
	if (TI) TI=0;					//If interrupt is because of transmission clear TI and return
	if(RI)
	{
		name=SBUF;				//If interrupt is because of reception, copy into name from SBUF
		RI=0;					//Clear RI
		SBUF=name;				//Retransmit the received data
		lcddata(name);				//Display on LCD the received data
	}
}
void main()
{
	SCON=0x50;					//Initialize serial port in mode 1 and enable reception
	TH1=0xFD;					//Count in timer 1 for abaud rate of 9600
	EA=1;					//Enable global interrupt
	ES=1;						//Enable serial interrupt
	TMOD=0X20;				//Timer 1 in mode 2 for baud rate generation
	for(i=0;i<5000;i++);		//Wait for some time for LCD to stabilize when power-on.
	lcdcommand(0x38);			//Issue the command to initialize 16X2 LCD.
	for(i=0;i<25;i++);			//Wait for some time using software delay.
	lcdcommand(0x0F);			//Issue the command for display on, cursor on and cursor blinking.
	for(i=0;i<25;i++);			//Wait for some time using software delay.
	lcdcommand(0x01);			//Issue the command to clear display.
	for(i=0;i<25;i++);			//Wait for some time using software delay.
	lcdcommand(0x06);			//Issue the command to increment cursor position on every character written.
	for(i=0;i<25;i++);			//Wait for some time using software delay.
	lcdcommand(0x80);			//Issue the command to position the cursor on the first position on line 1.
	for(i=0;i<25;i++);			//Wait for some time using software delay.
	TR1=1;					//Switch on timer 1 run bit
	while(1)
	{
	}
}
